package game;
import java.awt.*;

public class Sprite
{
    protected Color col;
    private int n;
    protected double x = 0, y = 0;
    protected int width, height;
    private double speedX = 0, speedY = 0;

    public Sprite (int defWidth, int defHeight, int r, int g, int b)
    {
        final int maxColVal = 255;

        col = new Color(r,g,b);
        width = defWidth;
        height = defHeight;
    }

    public Point getPoint()
    {
        return new Point(x,y);
    }

    public Rectangle getRectangle()
    {
        return new Rectangle( x, y, x+width, y+height );
    }

    public int getWidth()
    {
        return width;
    }

    public int getHeight()
    {
        return height;
    }

    public int getMidOff()
    {
        return (int) (width+height)/4;
    }

    public void randPos(int maxX, int maxY)
    {
        x = (int) ( Math.random()* (maxX-this.width*2) ) +this.width;
        y = (int) ( Math.random()* (maxY-this.height*2) ) +this.height;
    }

    public void randSpeed(boolean randX, boolean randY)
    {
        final int maxSpeed = 15;
        if ( randX )
            speedX = (int) ( Math.random()* (maxSpeed*2) - maxSpeed );
        if ( randY )
       	    speedY = (int) ( Math.random()* (maxSpeed*2) - maxSpeed );
    }

    public void push(double pushX, double pushY)
    {
        speedX += pushX;
        speedY += pushY;
    }

    public void follow(int targetX, int targetY)
    {
        final double adjustSpeed = 2, maxSpeed = 7;

        if ( targetX < x )
        {
            if ( speedX-adjustSpeed > -maxSpeed )
                speedX -= adjustSpeed;
        }
        else if ( targetX > x )
        {
            if ( speedX+adjustSpeed < maxSpeed )
                speedX += adjustSpeed;
        }

        if ( targetY < y )
        {
            if ( speedY-adjustSpeed > -maxSpeed )
                speedY -= adjustSpeed;
        }
        else if ( targetY > y )
        {
            if ( speedY+adjustSpeed < maxSpeed )
                speedY += adjustSpeed;
        }

    }

    public void shift(Ground grnd)
    {
        RectState rectState = new RectState();
        rectState.checkState( grnd.getRectangle(), getRectangle() );

        if ( rectState.getIsOutLeft() )
            x = grnd.getRight()-width;
        else if ( rectState.getIsOutRight() )
            x = grnd.getLeft();

        if ( rectState.getIsOutTop() )
            y = grnd.getBottom()-height;
        else if ( rectState.getIsOutBottom() )
            y = grnd.getTop();
    }

    public boolean shiftDisappear(Ground grnd)
    {
        boolean rVal = false;
        RectState rectState = new RectState();

        rectState.checkStateDisappear( grnd.getRectangle(), getRectangle() );

        if ( rectState.getIsOutLeft() )
        {
            x = grnd.getRight();
            rVal = true;
        }
        else if ( rectState.getIsOutRight() )
        {
            x = grnd.getLeft()-width;
            rVal = true;
        }

        if ( rectState.getIsOutTop() )
        {
            y = grnd.getBottom()-height;
            rVal = true;
        }            
        else if ( rectState.getIsOutBottom() )
        {
            y = grnd.getTop();
             rVal = true;
        }

        return rVal;
    }

    public void bounce(Ground grnd)
    {
        RectState rectState = new RectState();
        rectState.checkState( grnd.getRectangle(), getRectangle() );

        if ( rectState.isOutHorizontal() )
        {
            speedX *= -1;
            if ( rectState.getIsOutLeft() )
                x = grnd.getLeft();
            else // if ( rectState.getIsOutRight() )
                x = grnd.getRight()-width;
        }
        if ( rectState.getIsOutTop() || rectState.getIsOutBottom() )
        {
            speedY *= -1;
            if ( rectState.getIsOutTop() )
                y = grnd.getTop();
            else // if ( rectState.getIsOutBottom() )
                y = grnd.getBottom()-height;
        }
    }

    public void position(Point pos)
    {
        x = pos.getX();
        y = pos.getY();
    }

    public void move(Ground grnd)
    {
        speedY += grnd.getGravity();
        x += speedX;
        y += speedY;
    }

    public void appear(Graphics graph, Ground grnd)
    {
        graph.setColor(col);
        graph.fillOval
        (
            (int) (x+grnd.getOffsetX()), (int) (y+grnd.getOffsetY()),
            width, height
        );
    }

    public int number()
    { // number of object
        return n;
    }
}

class cameraSpr extends Sprite
{
    private int viewRad;

    cameraSpr(Ground grnd)
    {
        super( grnd.getWidth()/2, grnd.getHeight()/2, 0, 0, 0 );

        viewRad = grnd.getWidth()/2;
        width = viewRad*2;
        height = viewRad*2;
    }

    public int midX()
    {
        return ( (int) (x-viewRad) );
    }

    public int midY()
    {
        return ( (int) (y-viewRad) );
    }

    public void appear(Graphics graph)
    {
        Rectangle rect = new Rectangle();

        rect.setLeft( (int) (x-viewRad) );
        rect.setTop( (int) (y-viewRad) );
        rect.setWidth( (int) (viewRad*2) );
        rect.setHeight( (int) (viewRad*2) );

        graph.setColor( new Color(0,0,0) );

        graph.clipRect (
            rect.getLeft(), rect.getTop(), rect.getWidth(), rect.getHeight() );
        graph.fillRect (
            rect.getLeft(), rect.getTop(), rect.getWidth(), rect.getHeight() );
    }

}